@@ -1,5 +1,5 @@
 #
-# Copyright 2021-2022 Canonical Ltd.
+# Copyright 2021-2023 Canonical Ltd.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -42,6 +42,8 @@ from .instance_config import InstanceConfiguration
 
 logger = logging.getLogger(__name__)
 
+# pylint: disable=too-many-lines
+
 
 def default_command_environment() -> Dict[str, Optional[str]]:
     """Provide default command environment dictionary.
@@ -60,9 +62,9 @@ def default_command_environment() -> Dict[str, Optional[str]]:
 
     :returns: Dictionary of environment key/values.
     """
-    return dict(
-        PATH="/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/snap/bin"
-    )
+    return {
+        "PATH": "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/snap/bin"
+    }
 
 
 def _check_deadline(
@@ -144,6 +146,9 @@ class BuilddBaseAlias(enum.Enum):
     BIONIC = "18.04"
     FOCAL = "20.04"
     JAMMY = "22.04"
+    KINETIC = "22.10"
+    LUNAR = "23.04"
+    DEVEL = "devel"
 
 
 class Snap(pydantic.BaseModel, extra=pydantic.Extra.forbid):
@@ -266,6 +271,10 @@ class BuilddBase(Base):
     ) -> None:
         """Ensure instance configuration is compatible.
 
+        As long as the config is not incompatible (via a mismatched compatibility tag),
+        then assume the instance is compatible. This assumption is done because the
+        config file may not exist or contain a tag while the set up is in progress.
+
         :raises BaseCompatibilityError: if instance is incompatible.
         :raises BaseConfigurationError: on other unexpected error.
         """
@@ -284,8 +293,8 @@ class BuilddBase(Base):
         except FileNotFoundError:
             return
 
-        # make the same assumption as above for empty configs
-        if config is None:
+        # make the same assumption as above for empty configs or configs without a tag
+        if config is None or config.compatibility_tag is None:
             return
 
         if config.compatibility_tag != self.compatibility_tag:
@@ -299,18 +308,55 @@ class BuilddBase(Base):
             "Instance is compatible with compatibility tag %r", config.compatibility_tag
         )
 
-    def _ensure_os_compatible(
+    def _ensure_setup_completed(
         self, *, executor: Executor, deadline: Optional[float]
     ) -> None:
-        """Ensure OS is compatible with Base.
+        """Ensure the instance was fully setup.
 
-        :raises BaseCompatibilityError: if instance is incompatible.
-        :raises BaseConfigurationError: on other unexpected error.
+        The last step of setting up an instance is to set the `setup` key in the
+        instance config file to True. This flag is used to verify that setup was
+        completed for the instance.
+
+        :raises BaseCompatibilityError: If setup was not completed.
         """
+        _check_deadline(deadline)
+
+        try:
+            config = InstanceConfiguration.load(
+                executor=executor,
+                config_path=self.instance_config_path,
+            )
+        except ValidationError as error:
+            raise BaseCompatibilityError(
+                reason="failed to parse instance configuration file",
+            ) from error
+        except FileNotFoundError as error:
+            raise BaseCompatibilityError(
+                reason="failed to find instance config file",
+            ) from error
+
+        if config is None:
+            raise BaseCompatibilityError(reason="instance config is empty")
+
+        if not config.setup:
+            raise BaseCompatibilityError(reason="instance is marked as not setup")
+
+        logger.debug("Instance has already been setup.")
+
+    def _get_os_release(
+        self, *, executor: Executor, deadline: Optional[float]
+    ) -> Dict[str, str]:
+        """Get the OS release information from an instance's /etc/os-release.
+
+        :param executor: Executor to get OS release from.
+        :param deadline: Optional time.time() deadline.
+
+        :returns: Dictionary of key-mappings found in os-release.
+        """
+        _check_deadline(deadline)
         try:
             # Replace encoding errors if it somehow occurs with utf-8. This
             # doesn't need to be perfect for checking compatibility.
-            _check_deadline(deadline)
             proc = executor.execute_run(
                 command=["cat", "/etc/os-release"],
                 capture_output=True,
@@ -325,7 +371,17 @@ class BuilddBase(Base):
                 details=errors.details_from_called_process_error(error),
             ) from error
 
-        os_release = parse_os_release(proc.stdout)
+        return parse_os_release(proc.stdout)
+
+    def _ensure_os_compatible(
+        self, *, executor: Executor, deadline: Optional[float]
+    ) -> None:
+        """Ensure OS is compatible with Base.
+
+        :raises BaseCompatibilityError: if instance is incompatible.
+        :raises BaseConfigurationError: on other unexpected error.
+        """
+        os_release = self._get_os_release(executor=executor, deadline=deadline)
 
         os_name = os_release.get("NAME")
         if os_name != "Ubuntu":
@@ -335,6 +391,15 @@ class BuilddBase(Base):
 
         compat_version_id = self.alias.value
         version_id = os_release.get("VERSION_ID")
+
+        if compat_version_id == BuilddBaseAlias.DEVEL.value:
+            logger.debug(
+                "Ignoring OS version mismatch for %r because base is %r.",
+                version_id,
+                compat_version_id,
+            )
+            return
+
         if version_id != compat_version_id:
             raise BaseCompatibilityError(
                 reason=(
@@ -367,6 +432,10 @@ class BuilddBase(Base):
         of the installed dependencies required for subsequent use by the
         application.
 
+        The state of the setup is tracked in the instance config. If the setup is
+        interrupted, the state can be checked so that a partially setup instance
+        can be discarded.
+
         Setup may be called more than once in a given instance to refresh/update
         the environment.
 
@@ -393,6 +462,7 @@ class BuilddBase(Base):
         else:
             deadline = None
 
+        self._update_setup_status(executor=executor, deadline=deadline, status=False)
         self._ensure_os_compatible(executor=executor, deadline=deadline)
         self._ensure_instance_config_compatible(executor=executor, deadline=deadline)
         self._disable_automatic_apt(executor=executor, deadline=deadline)
@@ -400,7 +470,7 @@ class BuilddBase(Base):
         self._setup_wait_for_system_ready(
             executor=executor, deadline=deadline, retry_wait=retry_wait
         )
-        self._setup_instance_config(executor=executor, deadline=deadline)
+        self._update_compatibility_tag(executor=executor, deadline=deadline)
         self._setup_hostname(executor=executor, deadline=deadline)
         self._setup_resolved(executor=executor, deadline=deadline)
         self._setup_networkd(executor=executor, deadline=deadline)
@@ -412,6 +482,7 @@ class BuilddBase(Base):
         self._disable_and_wait_for_snap_refresh(executor=executor, deadline=deadline)
         self._setup_snapd_proxy(executor=executor, deadline=deadline)
         self._install_snaps(executor=executor, deadline=deadline)
+        self._update_setup_status(executor=executor, deadline=deadline, status=True)
 
     def warmup(
         self,
@@ -443,8 +514,13 @@ class BuilddBase(Base):
         else:
             deadline = None
 
+        self._ensure_setup_completed(executor=executor, deadline=deadline)
         self._ensure_os_compatible(executor=executor, deadline=deadline)
+
+        # XXX: checking the compatibility_tag should be much more strict when called
+        # by warmup (warmup will continue if the compatibility tag is missing or none!)
         self._ensure_instance_config_compatible(executor=executor, deadline=deadline)
+
         self._setup_wait_for_system_ready(
             executor=executor, deadline=deadline, retry_wait=retry_wait
         )
@@ -612,6 +688,14 @@ class BuilddBase(Base):
             file_mode="0644",
         )
 
+        # devel images should use the devel repository
+        if self.alias == BuilddBaseAlias.DEVEL:
+            self._update_apt_sources(
+                executor=executor,
+                deadline=deadline,
+                codename=BuilddBaseAlias.DEVEL.value,
+            )
+
         try:
             _check_deadline(deadline)
             _execute_run(executor, ["apt-get", "update"], verify_network=True)
@@ -685,16 +769,6 @@ class BuilddBase(Base):
                 details=errors.details_from_called_process_error(error),
             ) from error
 
-    def _setup_instance_config(
-        self, *, executor: Executor, deadline: Optional[float]
-    ) -> None:
-        InstanceConfiguration.update(
-            executor=executor,
-            data={"compatibility_tag": self.compatibility_tag},
-            config_path=self.instance_config_path,
-        )
-        _check_deadline(deadline)
-
     def _setup_networkd(self, *, executor: Executor, deadline: Optional[float]) -> None:
         """Configure networkd and start it.
 
@@ -912,6 +986,89 @@ class BuilddBase(Base):
             )
             sleep(retry_wait)
 
+    def _update_apt_sources(
+        self, *, executor: Executor, deadline: Optional[float], codename: str
+    ) -> None:
+        """Update the codename in the apt source config files.
+
+        :param executor: Executor for target container.
+        :param deadline: Optional time.time() deadline.
+        :param codename: New codename to use in apt source config files (i.e. 'lunar')
+        """
+        _check_deadline(deadline)
+
+        apt_source = "/etc/apt/sources.list"
+        apt_source_dir = "/etc/apt/sources.list.d/"
+
+        # get the current ubuntu codename
+        os_release = self._get_os_release(executor=executor, deadline=deadline)
+        version_codename = os_release.get("VERSION_CODENAME")
+        logger.debug("updating apt sources from %r to %r", version_codename, codename)
+
+        # replace all occurrences of the codename in the `sources.list` file
+        sed_command = ["sed", "-i", f"s/{version_codename}/{codename}/g"]
+        try:
+            _execute_run(executor, sed_command + [apt_source])
+        except subprocess.CalledProcessError as error:
+            raise BaseConfigurationError(
+                brief=f"Failed to update {apt_source!r}.",
+                details=errors.details_from_called_process_error(error),
+            ) from error
+
+        # running `find` and `sed` as two separate calls may appear unoptimized,
+        # but these shell commands will pass through `shlex.join()` before being
+        # executed, which means one-liners like `find -exec sed` or
+        # `find | xargs sed` cannot be used
+
+        try:
+            additional_source_files = _execute_run(
+                executor,
+                ["find", apt_source_dir, "-type", "f", "-name", "*.list"],
+                text=True,
+            ).stdout.strip()
+        except subprocess.CalledProcessError as error:
+            raise BaseConfigurationError(
+                brief=f"Failed to find apt source files in {apt_source_dir!r}.",
+                details=errors.details_from_called_process_error(error),
+            ) from error
+
+        # if there are config files in `sources.list.d/`, then update them
+        if additional_source_files:
+            try:
+                _execute_run(executor, sed_command + [apt_source_dir + "*.list"])
+            except subprocess.CalledProcessError as error:
+                raise BaseConfigurationError(
+                    brief=f"Failed to update apt source files in {apt_source_dir!r}.",
+                    details=errors.details_from_called_process_error(error),
+                ) from error
+
+    def _update_compatibility_tag(
+        self, *, executor: Executor, deadline: Optional[float]
+    ) -> None:
+        """Update the compatibility_tag in the instance config."""
+        InstanceConfiguration.update(
+            executor=executor,
+            data={"compatibility_tag": self.compatibility_tag},
+            config_path=self.instance_config_path,
+        )
+        _check_deadline(deadline)
+
+    def _update_setup_status(
+        self, *, executor: Executor, deadline: Optional[float], status: bool
+    ) -> None:
+        """Update the instance config to indicate the status of the setup.
+
+        :param executor: Executor for target container.
+        :param deadline: Optional time.time() deadline.
+        :param status: True if the setup is complete, False otherwise.
+        """
+        InstanceConfiguration.update(
+            executor=executor,
+            data={"setup": status},
+            config_path=self.instance_config_path,
+        )
+        _check_deadline(deadline)
+
     def wait_until_ready(
         self,
         *,
