@@ -39,3 +39,17 @@ class BaseCompatibilityError(ProviderError):
         resolution = "Clean incompatible instance and retry the requested operation."
 
         super().__init__(brief=brief, details=details, resolution=resolution)
+
+
+class NetworkError(ProviderError):
+    """Network error when configuring the base."""
+
+    def __init__(self) -> None:
+        brief = "A network related operation failed in a context of no network access."
+        # XXX Facundo 2022-12-13: need to improve the URL here once
+        # we have the online docs updated
+        resolution = (
+            "Verify that the environment has internet connectivity; "
+            "see https://craft-providers.readthedocs.io/ for further reference."
+        )
+        super().__init__(brief=brief, resolution=resolution)
