@@ -175,7 +175,7 @@ class Multipass:
         if cpus is not None:
             command.extend(["--cpus", cpus])
         if mem is not None:
-            command.extend(["--mem", mem])
+            command.extend(["--memory", mem])
         if disk is not None:
             command.extend(["--disk", disk])
 
@@ -334,7 +334,6 @@ class Multipass:
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE,
         ) as proc:
-
             # Should never happen, but mypy/pyright makes noise.
             assert proc.stdout is not None
             assert proc.stderr is not None
